      PROGRAM READEX
*
* This program converts a number of ISAMS level 3A files from binary
* to ASCII format and uses the following subroutines:
*
*    RBOPEN - Open a readable link to the input binary file
*    RBREAD - Read Binary record from the input file
*    RBSFDU - Read Binary SFDU record
*    RBLABL - Read Binary label record
*    RBMETA - Read Binary meta-data part of a data record
*    RBDATA - Read Binary data part of data file record
*    RBPARM - Read Binary data part of parameter file record     
*    WAOPEN - Open a writable link to the output ascii file
*    WAWRIT - Write Ascii record to the output file
*    WASFDU - Write Ascii SFDU record
*    WALABL - Write Ascii label record
*    WAMETA - Write Ascii meta-data part of a data record
*    WADATA - Write Ascii data part of data file record
*    WAPARM - Write Ascii data part of parameter file record
*
* The structure of the main controlling part of the program is shown
* below, with the called subroutine names in brackets:
*
*----------------------------------------------------------------------- 
*    For each pair of filenames in 'filename.dat' (GETFIL)...
*       Open the files in the correct format (RBOPEN and WAOPEN)
*
*       Convert the SFDU record from Binary to Ascii (RBSFDU and WASFDU)
*
*       Convert the label record from Binary to Ascii (RBLABL and WALABL)
*
*       For each data record...
*          Read in meta-data part of record (RBMETA)
*
*          If data file, then read data values (RBDATA)
*          Elseif parameter file, then read parameter values (RBPARM)
*
*          Write out meta-data part of record (WAMETA)
*
*          If data file, then write data values (WADATA)
*          Elseif parameter file, then write parameter values (WAPARM)
*       Next data record
*
*       Close files
*    Next pair of files
*-----------------------------------------------------------------------
*
* Local Variables...
*   INFILE - Input (Binary) filename
*   OUTFIL - Output (ASCII) filename
*   FSTOP  - Flag showing whether to stop converting (eg. end of file list)
*   FEXIST - Flag showing whether input file exists
*
      INCLUDE 'misc.inc'
      INCLUDE 'labl.inc'

      CHARACTER INFILE*132, OUTFIL*132
      LOGICAL   FSTOP, FEXIST
      INTEGER   I
*
* First, loop through the list getting the next two filenames...
*
   10 CONTINUE
         CALL GETFIL (INFILE,OUTFIL,FSTOP,FEXIST)
         IF (.NOT. FSTOP) THEN
*
* ...open the files...
*
            IF (FEXIST) THEN

               CALL RBOPEN (INFILE)
               CALL WAOPEN (OUTFIL)
*
* Loop through the file reading binary and writing ascii...
*
               CALL RBSFDU
               CALL WASFDU

               CALL RBLABL
               CALL WALABL

               IF (NCONTS .NE. 0) THEN
                  PRINT *,'* Warning - Number of label continuation'
                  PRINT *,'*           records is non-zero. These have'
                  PRINT *,'*           not yet been implemented into'
                  PRINT *,'*           this program, since we have not'
                  PRINT *,'*           seen any files containing them.'
                  PRINT *,'*           The continuation label records'
                  PRINT *,'*           will be ignored!'

                  DO 20, I = 1, NCONTS
                     CALL RBREAD (BINREC)
   20             CONTINUE
               ENDIF

               DO 30, I = 1, NPRECS - NCONTS - 1

                  CALL RBMETA

                  IF (LEVEL .EQ. '3AL' .OR. LEVEL .EQ. '3AT') THEN
                     CALL RBDATA
                  ELSEIF (LEVEL .EQ. '3LP' .OR. LEVEL .EQ. '3TP') THEN
                     CALL RBPARM
                  ENDIF                    

                  CALL WAMETA

                  IF (LEVEL .EQ. '3AL' .OR. LEVEL .EQ. '3AT') THEN
                     CALL WADATA
                  ELSEIF (LEVEL .EQ. '3LP' .OR. LEVEL .EQ. '3TP') THEN
                     CALL WAPARM
                  ENDIF                    

   30          CONTINUE
*
* When end of file reached, close files...
*
               CLOSE (UNIT=ASCLUN)
               CLOSE (UNIT=BINLUN)

            ELSE
               PRINT *,'* Error - Input file does not exist!'
            ENDIF
*
* ...and get next two filenames from the list...
*
            GOTO 10
         ENDIF

      END
